#!/bin/sh
# Copyright © 2023 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
#
# This file is part of MicroBenchmark.
#
# MicroBenchmark is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 3 of
# the License, or (at your option) any later version.
#
# MicroBenchmark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with MicroBenchmark.  If not, see
# <http://www.gnu.org/licenses/>.
set -e

: ${VERSION:="0.0"}
: ${ROUNDS:="1"}
: ${BUILDDIR:="$(pwd)/build"}
: ${RUN_EXAMPLES:=no}
: ${REGEN_DOC_EXAMPLES:=no}
: ${CONFIGURE_FLAGS:=""}

release_env='build-aux/guix.scm'
maint_env='build-aux/guix-maintainer.scm'
extra="--ad-hoc wget"
at_env="guix environment --pure -l ${maint_env} ${extra} --"

do_clean () {
    if [ "x${CLEAN-no}" != xno ]; then
        test -e build && chmod 755 -R build
        test -e inst && chmod 755 -R inst
        test -e .git && git clean -xfd
    fi
}

tarball="$1"
shift 2>/dev/null || echo "No tarball provided"
if [ ! -e "${tarball}" ]; then
    echo "Provided tarball ${tarball} does not exist, building from git."
    tarball=""
fi

do_clean
if [ -n "${BUILD_TARBALL+yes}" ]; then
    sv="https://git.savannah.gnu.org/gitweb"

    time ${at_env} bash <<EOF
RUN_EXAMPLES="${RUN_EXAMPLES}"
export RUN_EXAMPLES
REGEN_DOC_EXAMPLES="${REGEN_DOC_EXAMPLES}"
export REGEN_DOC_EXAMPLES
BUILDDIR="${BUILDDIR}"
export BUILDDIR
exec build-aux/build.sh ${CONFIGURE_FLAGS}
EOF
    tarball="${BUILDDIR}/mbenchmark-${VERSION}.tar.gz"
fi

toolchains="gcc-toolchain clang-toolchain"
if [ -n "${ALL_TOOLCHAINS+useall}" ]; then
    gcc_versions=""
    for v in $(seq 4 12); do
        gcc_versions="${gcc_versions} gcc-toolchain@${v}"
    done
    clang_versions=""
    for v in $(seq 6 15); do
        clang_versions="${clang_versions} clang-toolchain@${v}"
    done
    toolchains="${gcc_versions} ${clang_versions}"
fi

guile_versions="guile@2.2 guile@3.0"

for guile in ${guile_versions}; do
    for toolchain in ${toolchains}; do
        if [ -z "${tarball}" ]; then
            guix build --file="${maint_env}" \
                 --with-git-url=mbenchmark="$(pwd)" \
                 --with-c-toolchain=mbenchmark="${toolchain}" \
                 --with-input=guile="${guile}" \
                 --keep-failed
        else
            guix build --with-source=mbenchmark="${tarball}" \
                 --file="${release_env}" \
                 --rounds="${ROUNDS}" \
                 --with-c-toolchain=mbenchmark="${toolchain}" \
                 --with-input=guile="${guile}" \
                 --keep-failed
        fi
    done
done
