# mbenchmark.m4 --- Build configuration for MicroBenchmark.
#
# Copyright © 2023 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
#
# This file is free software; the author gives unlimited permission to
# copy and/or distribute it, with or without modifications, as long as
# this notice is preserved.
#
# serial 1
AC_DEFUN([MICRO_BENCHMARK_CHECK_C_FLAGS],
 [micro_benchmark_cppflags=
  micro_benchmark_libs=
  micro_benchmark_ltlibs=
  AC_ARG_WITH([micro-benchmark],
   [AS_HELP_STRING([--with-micro-benchmark=DIR],
     [Select MicroBenchmark installation])],
   [micro_benchmark_selected=$withval],
   [micro_benchmark_selected=check])

  AS_IF([test "x$micro_benchmark_selected" != xno],
   [oldLIBS=$LIBS
    micro_benchnmark_test_ok=
    dnl TODO: Use iteration
    mb_lib=
    AS_IF([test "x$micro_benchmark_selected" != xcheck],
     [LIBS="-L $micro_benchmark_selected/lib
      mb_lib="$micro_benchmark_selected/lib/libmbenchmark.la"],
     [test -e "$libdir/libmbenchmark.la"],
     [LIBS="-L $libdir"
      mb_lib="$libdir/libmbenchmark.la"],
     [test -e "$exec_prefix/lib/libmbenchmark.la"],
     [LIBS="-L $exec_prefix/lib"
      mb_lib="$exec_prefix/lib/libmbenchmark.la"],
     [test -e "$prefix/lib/libmbenchmark.la"],
     [LIBS="-L $prefix/lib"
      mb_lib="$prefix/lib/libmbenchmark.la"],
     [test -e "/opt/mbenchmark/lib/libmbenchmark.la"],
     [LIBS="-L /opt/mbenchmark/lib"
      mb_lib="/opt/mbenchmark/lib/libmbenchmark.la"],
     [test -e "$libdir/libmbenchmark.so"],
     [LIBS="-L $libdir"],
     [test -e "$exec_prefix/lib/libmbenchmark.so"],
     [LIBS="-L $exec_prefix/lib"],
     [test -e "$prefix/lib/libmbenchmark.so"],
     [LIBS="-L $prefix/lib"],
     [test -e "/opt/mbenchmark/lib/libmbenchmark.so"],
     [LIBS="-L /opt/mbenchmark/lib"])

    AC_LANG_PUSH([C])
    AC_CHECK_LIB([mbenchmark], [micro_benchmark_init],
      [AC_LINK_IFELSE([AC_LANG_PROGRAM([[
#include <mbenchmark/all.h>
        ]],
        [[
micro_benchmark_init ();
micro_benchmark_suite suite = micro_benchmark_suite_create ("test");
        ]])],
        [micro_benchmark_test_ok=yes],
        [micro_benchmark_test_ok=no])],
      [micro_benchmark_test_ok=no])
    AC_LANG_POP([C])

    AC_MSG_CHECKING([whether MicroBenchmark is available])
    AS_IF([test "x$micro_benchmark_test_ok" = xyes],
      [micro_benchmark_found=yes
       AC_MSG_RESULT([yes])
       micro_benchmark_cppflags=$CPPFLAGS
       micro_benchmark_libs=$LIBS
       AS_IF([test -e "$mb_lib"], [micro_benchmark_ltlibs="$mb_lib"])],
      [test "x$micro_benchmark_selected" = xcheck],
      [micro_benchmark_found=no
       AC_MSG_RESULT([not found])],
      [micro_benchmark_found=no
       AC_MSG_FAILURE([MicroBenchmark at $micro_benchmark_selected does not work])
      ])

    LIBS=$oldLIBS
    CPPFLAGS=$oldCPPFLAGS],
   [micro_benchmark_found=no])
 ])

dnl XXX parameters
AC_DEFUN([MICRO_BENCHMARK_CHECK_C],
 [AC_REQUIRE([MICRO_BENCHMARK_CHECK_C_FLAGS])
  m4_ifdef([AM_CONDITIONAL],
    [AM_CONDITIONAL([HAVE_MICRO_BENCHMARK],
      [test "x$micro_benchmark_found" = xyes])])
  AS_IF([test "x$micro_benchmark_found" = xyes],
   [AC_DEFINE([HAVE_MICRO_BENCHMARK], [1], [MicroBenchmark availability])])
  AC_SUBST([MICRO_BENCHMARK_CFLAGS], [$micro_benchmark_cppflags])
  AC_SUBST([MICRO_BENCHMARK_LIBS], [$micro_benchmark_libs])
  AC_SUBST([MICRO_BENCHMARK_LTLIBS], [$micro_benchmark_ltlibs])
 ])

AC_DEFUN([MICRO_BENCHMARK_CHECK_CXX_FLAGS],
 [AC_REQUIRE([MICRO_BENCHMARK_CHECK_C_FLAGS])
  AS_IF([test "x$micro_benchmark_found" = xyes],
   [oldLIBS=$LIBS
    oldCPPFLAGS=$CPPFLAGS
    CPPFLAGS=$micro_benchmark_cppflags
    LIBS=
    micro_benchmark_cxx_found=no
    mb_lib=
    AS_IF([test "x$micro_benchmark_selected" != xcheck],
     [AS_IF([test -e $micro_benchmark_selected/lib/mbenchmark],
       [LIBS="-L $micro_benchmark_selected/lib/mbenchmark
        mb_lib="$micro_benchmark_selected/lib/mbenchmark/libmbenchmark-c++.la"],
       [LIBS="-L $micro_benchmark_selected/lib
        mb_lib="$micro_benchmark_selected/lib/libmbenchmark-c++.la"
       ])],
     [test -e "$libdir/mbenchmark/libmbenchmark-c++.la"],
     [LIBS="-L $libdir/mbenchmark"
      mb_lib="$libdir/mbenchmark/libmbenchmark-c++.la"],

     [test -e "$libdir/libmbenchmark-c++.la"],
     [LIBS="-L $libdir"
      mb_lib="$libdir/libmbenchmark.la"],

     [test -e "$exec_prefix/lib/mbenchmark/libmbenchmark-c++.la"],
     [LIBS="-L $exec_prefix/lib/mbenchmark"
      mb_lib="$exec_prefix/lib/mbenchmark/libmbenchmark-c++.la"],

     [test -e "$exec_prefix/lib/libmbenchmark-c++.la"],
     [LIBS="-L $exec_prefix/lib"
      mb_lib="$exec_prefix/lib/libmbenchmark-c++.la"],

     [test -e "$prefix/lib/mbenchmark/libmbenchmark-c++.la"],
     [LIBS="-L $prefix/lib/mbenchmark"
      mb_lib="$prefix/lib/mbenchmark/libmbenchmark-c++.la"],

     [test -e "$prefix/lib/libmbenchmark-c++.la"],
     [LIBS="-L $prefix/lib"
      mb_lib="$prefix/lib/libmbenchmark-c++.la"],

     [test -e "/opt/mbenchmark/lib/mbenchmark/libmbenchmark-c++.la"],
     [LIBS="-L /opt/mbenchmark/lib/mbenchmark"
      mb_lib="/opt/mbenchmark/lib/mbenchmark/libmbenchmark-c++.la"],

     [test -e "/opt/mbenchmark/lib/libmbenchmark-c++.la"],
     [LIBS="-L /opt/mbenchmark/lib"
      mb_lib="/opt/mbenchmark/lib/libmbenchmark-c++.la"],

     [test -e "$libdir/mbenchmark/libmbenchmark-c++.so"],
     [LIBS="-L $libdir/mbenchmark"],
     [test -e "$libdir/libmbenchmark-c++.so"],
     [LIBS="-L $libdir"],
     [test -e "$exec_prefix/lib/mbenchmark/libmbenchmark-c++.so"],
     [LIBS="-L $exec_prefix/lib/mbenchmark"],
     [test -e "$exec_prefix/lib/libmbenchmark-c++.so"],
     [LIBS="-L $exec_prefix/lib"],
     [test -e "$prefix/lib/mbenchmark/libmbenchmark.so"],
     [LIBS="-L $prefix/lib/mbenchmark"],
     [test -e "$prefix/lib/libmbenchmark.so"],
     [LIBS="-L $prefix/lib"],
     [test -e "/opt/mbenchmark/lib/mbenchmark/libmbenchmark-c++.so"],
     [LIBS="-L /opt/mbenchmark/lib/mbenchmark"],
     [test -e "/opt/mbenchmark/lib/libmbenchmark-c++.so"],
     [LIBS="-L /opt/mbenchmark/lib"])

    AC_LANG_PUSH([C++])
    AC_CHECK_LIB([mbenchmark-c++], [micro_benchmark::main],
      [micro_benchmark_cxx_found=yes],
      [micro_benchmark_cxx_found=no])

    AC_MSG_CHECKING([MicroBenchmark C++ library])
    micro_benchmark_test_ok=
    AS_IF([test "x$micro_benchmark_cxx_found" = xyes],
     [AC_LINK_IFELSE([AC_LANG_PROGRAM([[#include <mbenchmark/all.hpp>
       ]], [[micro_benchmark::suite suite ("test");]])],
       [micro_benchmark_test_ok=yes],
       [micro_benchmark_test_ok=no])])
    AC_LANG_POP([C++])

    AS_IF([test "x$micro_benchmark_test_ok" = xyes],
     [micro_benchmark_cxx_found=yes
      micro_benchmark_cxxlibs=$LIBS
      AS_IF([test "x$mb_lib" != x && test -e "$mb_lib"],
       [micro_benchmark_cxxltlibs=$mb_lib])
      AC_MSG_RESULT([ok])],
     [test "x$micro_benchmark_selected" = xcheck],
     [micro_benchmark_cxx_found=no
      AC_MSG_RESULT([not found])],
     [micro_benchmark_cxx_found=no
      AC_MSG_FAILURE([not found])])

    LIBS=$oldLIBS
    CPPFLAGS=$oldCPPFLAGS],
   [micro_benchmark_cxx_found=no
    AS_IF([test "x$micro_benchmark_selected" = xcheck], [],
      [test "x$micro_benchmark_selected" = no], [],
      [AC_MSG_ERROR([MicroBenchmark not found.])])
   ])
 ])

dnl XXX parameters
AC_DEFUN([MICRO_BENCHMARK_CHECK_CXX],
 [AC_REQUIRE([MICRO_BENCHMARK_CHECK_CXX_FLAGS])
  m4_ifdef([AM_CONDITIONAL],
    [AM_CONDITIONAL([HAVE_MICRO_BENCHMARK_CXX],
      [test "x$micro_benchmark_cxx_found" = xyes])])
  AS_IF([test "x$micro_benchmark_found" = xyes],
   [AC_DEFINE([HAVE_MICRO_BENCHMARK_CXX], [1],
     [MicroBenchmark C++ availability])])
  AC_SUBST([MICRO_BENCHMARK_CXXFLAGS], [$micro_benchmark_cppflags])
  AC_SUBST([MICRO_BENCHMARK_CXXLIBS], [$micro_benchmark_cxxlibs])
  AC_SUBST([MICRO_BENCHMARK_CXXLTLIBS], [$micro_benchmark_cxxltlibs])
 ])

dnl XXX parameters
AC_DEFUN([MICRO_BENCHMARK_CHECK_GUILE],
 [dnl TODO: Better checks
  GUILE_MODULE_REQUIRED_EXPORT([mbenchmark], [main])
  GUILE_MODULE_REQUIRED_EXPORT([mbenchmark log], [set-log-level!])
  GUILE_MODULE_REQUIRED_EXPORT([mbenchmark report], [report?])
  GUILE_MODULE_REQUIRED_EXPORT([mbenchmark state], [keep-running?])
  GUILE_MODULE_REQUIRED_EXPORT([mbenchmark suite], [run-suite!])
  GUILE_MODULE_REQUIRED_EXPORT([mbenchmark test], [register-test!])
 ])
