/* constraints.h --- MicroBenchmark library. */
/*
 * Copyright © 2023 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
 *
 * This file is part of MicroBenchmark.
 *
 * MicroBenchmark is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * MicroBenchmark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with MicroBenchmark.  If not, see
 * <http://www.gnu.org/licenses/>.
 */
#ifndef MICRO_BENCHMARK_INTERNAL_TEST_CONSTRAINTS_H
#define MICRO_BENCHMARK_INTERNAL_TEST_CONSTRAINTS_H

#include "mbenchmark/common.h"
#include "mbenchmark/test.h"
#include "internal/test/constraints/iteration.h"
#include "internal/test/constraints/meter.h"
#include "internal/test/constraints/size.h"
#include "internal/test/constraints/time.h"

#include <stddef.h>

/* *INDENT-OFF* */
MICRO_BENCHMARK_BEGIN_DECLS
/* *INDENT-ON* */

typedef struct micro_benchmark_test_constraints_
{
  mbtest_size_constraint_ size_constraint;
  mbtest_iteration_constraint_ iteration_constraint;
  mbtest_time_constraint_ time_constraint;
  mbtest_meter_constraint_ meter_constraint;
} mbtest_constraints_;

void micro_benchmark_test_constraints_init_ (mbtest_constraints_ *);

void micro_benchmark_test_constraints_cleanup_ (mbtest_constraints_ *);

void
micro_benchmark_test_constraints_add_dimension_ (mbtest_constraints_ *tc,
                                                 size_t num_sizes,
                                                 const size_t *sizes);

size_t
micro_benchmark_test_constraints_dimensions_ (const mbtest_constraints_ *tc);

void
micro_benchmark_test_constraints_get_dimension_ (const mbtest_constraints_
                                                 *tc, size_t dimension,
                                                 size_t *n_sizes,
                                                 const size_t **sizes);

void
micro_benchmark_test_constraints_skip_iterations_ (mbtest_constraints_ *tc,
                                                   size_t iterations_to_skip);

size_t
micro_benchmark_test_constraints_iterations_to_skip_ (const
                                                      mbtest_constraints_
                                                      *tc);

void
micro_benchmark_test_constraints_limit_iterations_ (mbtest_constraints_ *tc,
                                                    size_t min_iterations,
                                                    size_t max_iterations);

size_t
micro_benchmark_test_constraints_min_iterations_ (const mbtest_constraints_
                                                  *tc);

size_t
micro_benchmark_test_constraints_max_iterations_ (const mbtest_constraints_
                                                  *tc);

void
micro_benchmark_test_constraints_limit_samples_ (mbtest_constraints_ *tc,
                                                 size_t min_iterations,
                                                 size_t max_iterations);

size_t
micro_benchmark_test_constraints_min_sample_iterations_ (const
                                                         mbtest_constraints_
                                                         *tc);


size_t
micro_benchmark_test_constraints_max_sample_iterations_ (const
                                                         mbtest_constraints_
                                                         *tc);

void
micro_benchmark_test_constraints_set_max_time_ (mbtest_constraints_ *tc,
                                                micro_benchmark_clock_time d);


micro_benchmark_clock_time
micro_benchmark_test_constraints_get_max_time_ (const mbtest_constraints_
                                                *tc);


void
micro_benchmark_test_constraints_set_chrono_ (mbtest_constraints_ *tc,
                                              micro_benchmark_clock_type type,
                                              micro_benchmark_chronometer_provider
                                              p);

void
micro_benchmark_test_constraints_set_custom_chrono_ (mbtest_constraints_ *tc,
                                                     micro_benchmark_clock_type,
                                                     micro_benchmark_meter m);

void
micro_benchmark_test_constraints_set_calculator_ (mbtest_constraints_ *tc,
                                                  micro_benchmark_custom_time_calculator);

void
micro_benchmark_test_constraints_add_meter_ (mbtest_constraints_ *tc,
                                             micro_benchmark_custom_meter m);

MICRO_BENCHMARK_END_DECLS
#endif
