/* meter.h --- MicroBenchmark library. */
/*
 * Copyright © 2023 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
 *
 * This file is part of MicroBenchmark.
 *
 * MicroBenchmark is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * MicroBenchmark is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with MicroBenchmark.  If not, see
 * <http://www.gnu.org/licenses/>.
 */
#ifndef MICRO_BENCHMARK_INTERNAL_TEST_CONSTRAINTS_METER_H
#define MICRO_BENCHMARK_INTERNAL_TEST_CONSTRAINTS_METER_H

#include "mbenchmark/common.h"
#include "mbenchmark/clock/type.h"

/* *INDENT-OFF* */
MICRO_BENCHMARK_BEGIN_DECLS
/* *INDENT-ON* */

typedef struct micro_benchmark_test_meter_constraint_
{
  /** Enable custom iteration chronometer.  */
  bool has_chronometer;
  /** Custom iteration clock type.  */
  micro_benchmark_clock_type chrono_type;
  /** Custom iteration chronometer provider.  */
  micro_benchmark_chronometer_provider chrono_provider;
  /** Custom iteration chronometer based on custom meter.  */
  micro_benchmark_meter chrono_tmpl;

  /** Enable custom chronometer stats calculator.  */
  bool has_custom_stats;
  /** Function called to aggregate the collected data.  */
  micro_benchmark_custom_time_calculator custom_stats;

  /** Enable custom sample meters.  */
  bool has_custom_meters;
  /** Number of custom sample meters provided.  */
  size_t num_custom_meters;
  /** Array of custom sample meters provided.  */
  micro_benchmark_custom_meter *custom_meters;
} mbtest_meter_constraint_;

MICRO_BENCHMARK_END_DECLS
#endif
